/* SIMulador Sistema PLAnetario, M.Alvarez 2018
 * Trabajo para el TFM
 * del Master de Astrofísica de la UAM
 * 
 * Copyright (c) 2018 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of simspla
 *
 * simspla is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * simspla is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with rebound.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#define VERSION "Extract_Sim v1.10"

#define GOOD_RETURN 0
#define BAD_RETURN 1

#define PI 3.14159265358979323846
#define MIN_INC 1.0e-8

#define G_AUDAYKG 1.488136116290668e-34 // AU^3/(kg day^2)
#define G G_AUDAYKG

#define FILEOUTd    "evol_d.dat"
#define FILEOUTdmed "evol_dmed.dat"
#define FILEOUTab   "evol_ab.dat"
#define FILEOUTei   "evol_ei.dat"
#define FILEOUTT    "evol_T.dat"
#define FILEOUTpa   "evol_pa.dat"
#define FILEOUToo   "evol_oo.dat"
#define FILEMIND    "mind.dat"
#define FILEGCON    "dg2.dat"

typedef struct { // componentes cartesianas 3d
    double x, y, z;
} Triplet;

typedef struct {
    double d;
    double lim;
    long step;
} MinDist;

typedef struct {
    Triplet pos;
    Triplet v;
    double hillr;  // radio de Hill (astro >0)
    double distm;  // distancia media al astro 0 (astro >0)
} StepData;

typedef struct {
    char *nombre;
    StepData *sd;
    double m;
} Body;

typedef struct {
    int nbodies;
    Body *astro;
    long ncycles;
    long nfcycle;
    long nsteps;
    long tsteps;
    long *step;
    double *msun;
    MinDist *mind;
} System;

typedef struct {
    double dist;    // distancia (real)
    double a;       // semieje mayor
    double e;       // excentricidad
    double T;       // periodo
    double i;       // inclinación
    double Omega;   // longitud del nodo ascendente
    double omega;   // argumento del perihelio
    double M;       // anomalía media
} Orbita;

/* Crea sistema planetario desde fichero definición, si ncycles==0 toma
 * ncycles, nfcycle, nsteps del fichero
 */
System * crea_planet_system_de_filedef(char *filedef, long ncycles, long nfcycle, long nsteps)
{
    System *s = NULL;
    char buf[301], *bufp;
    int i;
    FILE *f;
    int inisys = 0;
    long fnsave = 1;
    long fnsteps = 1;
    long fncycles = 1;
    long fnfcycle = 0;
    char *token;
    int indn1 = 0;
    long mem = 0;

    f = fopen(filedef, "r");
    if (f == NULL) {
        fprintf(stderr,"Error al abrir %s\n", filedef);
        return NULL;
    }

    while (fgets(buf, 300, f) != NULL) {
        if (buf[0] == '#' || buf[0] == '\0') continue;
        bufp = buf;
        while (*bufp == ' ') bufp++; // elimina blancos

        if (!strncmp(bufp,"SAVESTEP=",9)) fnsave = atol(&(bufp[9]));
        if (!strncmp(bufp,"NSTEPS=",7)) fnsteps = atol(&(bufp[7]));
        if (!strncmp(bufp,"NCYCLES=",8)) fncycles = atol(&(bufp[8]));
        if (!strncmp(bufp,"NFCYCLE=",8)) fnfcycle = atol(&(bufp[8]));

        if (!inisys && !strncmp(bufp,"SYSTEM,",7)) {
            s = (System *)malloc(sizeof(System));
            if (s == NULL) return NULL;
            mem += sizeof(System);
            token = strtok(&(bufp[7]), ","); // nombre
            token = strtok(NULL, ",");
            s->nbodies = atoi(token);
            if (ncycles > 0) {
                s->nsteps = nsteps;
                s->ncycles = ncycles;
                s->nfcycle = nfcycle;
            } else {
                s->nsteps = fnsteps / fnsave;
                s->ncycles = fncycles;
                s->nfcycle = fnfcycle;
            }
            s->tsteps = (s->ncycles - s->nfcycle) * s->nsteps;
            s->astro = (Body *)malloc(s->nbodies * sizeof(Body));
            if (s->astro == NULL) {
                free(s);
                return NULL;
            }
            mem += s->nbodies * sizeof(Body);
            s->step = (long *)malloc(s->tsteps * sizeof(long));
            if (s->step == NULL) {
                free(s->astro);
                free(s);
                return NULL;
            }
            mem += s->tsteps * sizeof(long);
            s->msun = (double *)malloc(s->tsteps * sizeof(double));
            if (s->msun == NULL) {
                free(s->step);
                free(s->astro);
                free(s);
                return NULL;
            }
            mem += s->tsteps * sizeof(double);
            s->mind = (MinDist *)malloc(s->nbodies * s->nbodies * sizeof(MinDist));
            if (s->mind == NULL) {
                free(s->msun);
                free(s->step);
                free(s->astro);
                free(s);
                return NULL;
            }
            mem += s->nbodies * s->nbodies * sizeof(MinDist);
            for (i=0; i<(s->nbodies*s->nbodies); i++) {
                s->mind[i].d = 1e99;
                s->mind[i].step = -1;
            }
            indn1 = 0;
            inisys = 1;
        }
    
        if (inisys && !strncmp(bufp,"BODY,",5)) {
            if (indn1 < s->nbodies) {
                token = strtok(&(bufp[5]), ",");
                s->astro[indn1].nombre = strdup(token);
                token = strtok(NULL, ","); // tipo
                token = strtok(NULL, ","); // estado
                token = strtok(NULL, ",");
                s->astro[indn1].m = atof(token);
                s->astro[indn1].sd = (StepData *)malloc(s->tsteps * sizeof(StepData));
                if (s->astro[indn1].sd == NULL) return NULL;
                //En buena lid habría que liberar memoria
                mem += s->tsteps * sizeof(StepData);
                indn1++;
            }
        }
    }
  
    if (indn1 != s->nbodies) {
        fprintf(stderr, "Error, sólo definidos %d astros de %d\n", indn1, s->nbodies);
        return NULL;
    }

    printf("Creado System, memoria estimada %ld\n", mem);

    return s;
}

/* Carga los datos desde los ficheros
 */
int carga_datos(System *s, char *datadir, int binary)
{
    FILE *f;
    char buf[301];
    char fname[101];
    long st;
    Triplet tp, tv;
    double hillr, distm;
    int i;
    long ncycle, indice;
    double d1, d2, d3, d4, msun, d6;

    for (ncycle=s->nfcycle; ncycle<s->ncycles; ncycle++) {
        for (i=0; i< s->nbodies; i++) {
            if (!binary) {
                snprintf(fname, 100, "%s/%s.%04ld.dat", datadir, s->astro[i].nombre, ncycle);
                f = fopen(fname, "r");
            } else {
                snprintf(fname, 100, "%s/%s.%04ld.bin", datadir, s->astro[i].nombre, ncycle);
                f = fopen(fname, "rb");
            }
            if (f == NULL) {
                fprintf(stderr, "Error al abrir %s\n", fname);
                return BAD_RETURN;
            }
  
            indice = s->nsteps * (ncycle - s->nfcycle);
            if (!binary) {
                while (fgets(buf, 300, f) != NULL) {
                    sscanf(buf,"%ld %le %le %le %le %le %le %le %le",
                           &st, &tp.x, &tp.y, &tp.z, &tv.x, &tv.y, &tv.z, &hillr, &distm);
                    if (indice < s->tsteps) {
                        if (i==0) s->step[indice] = st;
                        s->astro[i].sd[indice].pos = tp;
                        s->astro[i].sd[indice].v = tv;
                        s->astro[i].sd[indice].hillr = hillr;
                        s->astro[i].sd[indice].distm = distm;
                        indice++;
                    }
                }
            } else {
                while (fread(&st, sizeof(long), 1, f) == 1) {
                    fread(&tp, sizeof(Triplet), 1, f);
                    fread(&tv, sizeof(Triplet), 1, f);
                    fread(&hillr, sizeof(double), 1, f);
                    fread(&distm, sizeof(double), 1, f);
                    if (indice < s->tsteps) {
                        if (i==0) s->step[indice] = st;
                        s->astro[i].sd[indice].pos = tp;
                        s->astro[i].sd[indice].v = tv;
                        s->astro[i].sd[indice].hillr = hillr;
                        s->astro[i].sd[indice].distm = distm;
                        indice++;
                    }
                }
            }
            fclose(f);
        }
        snprintf(fname, 100, "%s/dglobal.%04ld.dat", datadir, ncycle);
        f = fopen(fname, "r");
        if (f == NULL) {
            fprintf(stderr, "Error al abrir %s\n", fname);
            return BAD_RETURN;
        }
        indice = s->nsteps * (ncycle - s->nfcycle);
        while (fgets(buf, 300, f) != NULL) {
            sscanf(buf,"%ld %le %le %le %le %le %le",
                   &st, &d1, &d2, &d3, &d4, &msun, &d6);
            if (indice < s->tsteps) {
                s->msun[indice] = msun;
                indice++;
            }
        }
        fclose(f);
    }
  
    return GOOD_RETURN;
}

/*void muestra_system(System *s)
{
    StepData *sd;
    int i;

    for (i=0; i<s->nbodies; i++) {
        sd = &(s->astro[i].sd);
        printf("%-20s %10ld ", s->astro[i].nombre, sd->step);
        //printf("%22.15e %22.15e %22.15e ", sd->pos.x, sd->pos.y, sd->pos.z);
        //printf("%22.15e %22.15e %22.15e\n", sd->v.x, sd->v.y, sd->v.z);
    }
    printf("\n");
}*/

/* Calcula arco coseno de num/denom
 */
static double acos2(double num, double denom, double disam) {
    double coseno;
    
    if (denom == 0.0) return 0.0;
    coseno = num / denom;
    if (coseno > -1.0 && coseno < 1.0) {
        if (disam >= 0.0)
            return acos(coseno);
        else
            return -acos(coseno);
    } else {
        return (coseno <= -1.0) ? PI : 0.0;
    }
}

/* Calcula datos orbitales desde posición y velocidad
 */
void calcula_orbita(Body *cb, Body *target, long step, Orbita *o)
{
    double mu;
    double dx, dy, dz, dvx, dvy, dvz;
    double dist, vel2, vc2, vdiff2, vr2;
    double ex, ey, ez, mm;
    double h, hx, hy, hz;
    double nx, ny, n;
    double pomega;
  
    if (target->sd[step].pos.x == 0.0) {
        o->dist = 0.0;
        o->a  = 0.0;
        o->e  = 0.0;
        o->T  = 0.0;
        o->i  = 0.0;
        o->Omega = 0.0;
        o->omega = 0.0;
        o->M = 0.0;
        return;
    }
  
    mu = G * (target->m + cb->m);
    dx = target->sd[step].pos.x - cb->sd[step].pos.x;
    dy = target->sd[step].pos.y - cb->sd[step].pos.y;
    dz = target->sd[step].pos.z - cb->sd[step].pos.z;
    dvx = target->sd[step].v.x - cb->sd[step].v.x;
    dvy = target->sd[step].v.y - cb->sd[step].v.y;
    dvz = target->sd[step].v.z - cb->sd[step].v.z;
    dist = sqrt(dx*dx + dy*dy + dz*dz);
    vel2 = dvx*dvx + dvy*dvy + dvz*dvz;
    vc2 = mu / dist;

    o->dist = dist;
  
    o->a = -mu / (vel2 - 2.0 * vc2); // semi-eje mayor

    vdiff2 = vel2 - vc2;
    vr2 = (dx*dvx + dy*dvy + dz*dvz);
    ex = (vdiff2*dx - vr2*dvx) / mu;
    ey = (vdiff2*dy - vr2*dvy) / mu;
    ez = (vdiff2*dz - vr2*dvz) / mu;

    o->e = sqrt(ex*ex + ey*ey +ez*ez); // excentricidad
  
    mm = o->a / fabs(o->a) * sqrt(fabs(mu / (o->a * o->a * o->a)));
  
    o->T = 2 * PI / mm; // periodo

    hx = (dy*dvz - dz*dvy);
    hy = (dz*dvx - dx*dvz);
    hz = (dx*dvy - dy*dvx);
    h = sqrt(hx*hx + hy*hy + hz*hz);
  
    o->i = (h == 0.0) ? 0.0 : acos(hz/h); // inclinación
  
    nx = -hy;
    ny = hx;
    n = sqrt(nx*nx + ny*ny);
  
    o->Omega = acos2(nx, n, ny); // longitud del nodo ascendente
  
    if (o->i < MIN_INC || o->i > (PI - MIN_INC)) {
        pomega = acos2(ex, o->e, ey);
        if (o->i < PI/2.0)
            o->omega = pomega - o->Omega; // argumento del perihelio
        else
            o->omega = o->Omega - pomega;
    } else{
        o->omega = acos2(nx*ex+ny*ey, n*o->e, ez);
    }
}

/* Calcula y graba datos orbitales
 */
int graba_datos_orbitas(System *s, char *procdir)
{
    FILE *fd, *fdmed, *fab, *fei, *fT, *fpa, *foo;
    char fname[101];
    int i;
    long step;
    Orbita o;
  
    snprintf(fname, 100, "%s/%s", procdir, FILEOUTd);
    fd = fopen(fname, "w");
    if (fd == NULL) {
        fprintf(stderr, "Error al crear %s\n", fname);
        return BAD_RETURN;
    }
    snprintf(fname, 100, "%s/%s", procdir, FILEOUTdmed);
    fdmed = fopen(fname, "w");
    if (fdmed == NULL) {
        fprintf(stderr, "Error al crear %s\n", fname);
        return BAD_RETURN;
    }
    snprintf(fname, 100, "%s/%s", procdir, FILEOUTab);
    fab = fopen(fname, "w");
    if (fab == NULL) {
        fprintf(stderr, "Error al crear %s\n", fname);
        return BAD_RETURN;
    }
    snprintf(fname, 100, "%s/%s", procdir, FILEOUTei);
    fei = fopen(fname, "w");
    if (fei == NULL) {
        fprintf(stderr, "Error al crear %s\n", fname);
        return BAD_RETURN;
    }
    snprintf(fname, 100, "%s/%s", procdir, FILEOUTT);
    fT = fopen(fname, "w");
    if (fT == NULL) {
        fprintf(stderr, "Error al crear %s\n", fname);
        return BAD_RETURN;
    }
    snprintf(fname, 100, "%s/%s", procdir, FILEOUTpa);
    fpa = fopen(fname, "w");
    if (fpa == NULL) {
        fprintf(stderr, "Error al crear %s\n", fname);
        return BAD_RETURN;
    }
    snprintf(fname, 100, "%s/%s", procdir, FILEOUToo);
    foo = fopen(fname, "w");
    if (foo == NULL) {
        fprintf(stderr, "Error al crear %s\n", fname);
        return BAD_RETURN;
    }

    for (step=0; step<s->tsteps; step++) {
        s->astro[0].m = s->msun[step];
        fprintf(fd, "%ld ", s->step[step]);
        fprintf(fab, "%ld ", s->step[step]);
        fprintf(fei, "%ld ", s->step[step]);
        fprintf(fT, "%ld ", s->step[step]);
        fprintf(fpa, "%ld ", s->step[step]);
        fprintf(foo, "%ld ", s->step[step]);
        for (i=1; i<s->nbodies; i++) {
            calcula_orbita(&(s->astro[0]), &(s->astro[i]), step, &o);
            fprintf(fd, "%22.15e ", o.dist);                        // distancia
            fprintf(fab, "%22.15e ", o.a);                          // semi-eje mayor
            fprintf(fab, "%22.15e ", o.a * sqrt(1 - (o.e * o.e)));  // semi-eje menor
            fprintf(fei, "%22.15e ", o.e);                          // excentricidad
            fprintf(fei, "%22.15e ", o.i);                          // inclinación
            fprintf(fT, "%22.15e ", o.T);                           // periodo
            fprintf(fpa, "%22.15e ", o.a * (1.0 - o.e));            // periastro
            fprintf(fpa, "%22.15e ", o.a * (1.0 + o.e));            // apoastro
            fprintf(foo, "%22.15e ", o.Omega);                      // longitud del nodo ascendente
            fprintf(foo, "%22.15e ", o.omega);                      // argumento del perihelio
        }
        fprintf(fd, "\n");
        fprintf(fab, "\n");
        fprintf(fei, "\n");
        fprintf(fT, "\n");
        fprintf(fpa, "\n");
        fprintf(foo, "\n");
    }

    for (step=0; step<s->tsteps; step++) {
        if ((step % s->nsteps) == 0) continue; // no hay dato en primer step de cada ciclo
        s->astro[0].m = s->msun[step];
        fprintf(fdmed, "%ld ", s->step[step]);
        for (i=1; i<s->nbodies; i++) {
            fprintf(fdmed, "%22.15e ", s->astro[i].sd[step].distm); // distancia media
        }
        fprintf(fdmed, "\n");
     }
 
    fclose(fd);
    fclose(fdmed);
    fclose(fab);
    fclose(fei);
    fclose(fT);
    fclose(fpa);
    fclose(foo);

    return GOOD_RETURN;
}

/* Calcula y graba datos de minima distancia
 * ojo! solo de los datos guardados
 */
int graba_mind(System *s, char *procdir)
{
    FILE *f;
    char fname[101];
    int i, j, nb;
    long step;
    StepData *sd1, *sd2;
    double distx, disty, distz, dist;
  
    snprintf(fname, 100, "%s/%s", procdir, FILEMIND);
    f = fopen(fname, "w");
    if (f == NULL) {
        fprintf(stderr, "Error al crear %s\n", fname);
        return BAD_RETURN;
    }

    nb = s->nbodies;
    for (i=0; i<nb; i++) {
        for (j=0; j<nb; j++) {
            if (i==j) continue;
            for (step=0; step<s->tsteps; step++) {
                sd1 = &(s->astro[i].sd[step]);
                sd2 = &(s->astro[j].sd[step]);
                if ((sd1->pos.x == 0.0) || (sd2->pos.x == 0.0)) continue;
                distx = sd1->pos.x - sd2->pos.x;
                disty = sd1->pos.y - sd2->pos.y;
                distz = sd1->pos.z - sd2->pos.z;
                dist = sqrt(distx * distx + disty * disty + distz * distz);
                if (dist < s->mind[i*nb+j].d) {
                    s->mind[i*nb+j].d = dist;
                    s->mind[i*nb+j].step = s->step[step];
                }
            }
        }
    }
  
    for (i=0; i<nb-1; i++) {
        for (j=i+1; j<nb; j++) {
            fprintf(f, "%d %d %10ld %22.15e\n", i, j, s->mind[i*nb+j].step, s->mind[i*nb+j].d);
        }
    }
  
    fclose(f);
  
    return GOOD_RETURN;
}

/* Calcula y graba datos de minima distancia total de la simulación leyendo los
 * ficheros mindist.ciclo.dat
 */
int graba_mind2(System *s, char *datadir, char *procdir)
{
    int i, j, nb;
    FILE *f;
    char buf[301];
    char fname[101];
    long ncycle;
    int a1, a2;
    double rt, rd, rlim;

    nb = s->nbodies;
    for (i=0; i<(nb*nb); i++) {
        s->mind[i].d = 1e99;
        s->mind[i].lim = 0;
        s->mind[i].step = -1;
    }

    for (ncycle=s->nfcycle; ncycle<s->ncycles; ncycle++) {
        snprintf(fname, 100, "%s/mindist.%04ld.dat", datadir, ncycle);
        f = fopen(fname, "r");
        if (f == NULL) {
            fprintf(stderr, "Error al abrir %s\n", fname);
            return BAD_RETURN;
        }
        while (fgets(buf, 300, f) != NULL) {
            sscanf(buf,"%d %d %le %le %le", &a1, &a2, &rt, &rd, &rlim);
            if ((a1 < 0) || (a1 >= nb)) continue;
            if ((a2 < 0) || (a2 >= nb)) continue;
            if (rd < s->mind[a1*nb+a2].d) {
                s->mind[a1*nb+a2].step = rt;
                s->mind[a1*nb+a2].d = rd;
                s->mind[a1*nb+a2].lim = rlim;
            }
        }
        fclose(f);
    }

    snprintf(fname, 100, "%s/%s", procdir, FILEMIND);
    f = fopen(fname, "w");
    if (f == NULL) {
        fprintf(stderr, "Error al crear %s\n", fname);
        return BAD_RETURN;
    }
    for (i=0; i<nb-1; i++) {
        for (j=i+1; j<nb; j++) {
            fprintf(f, "%d %d %10ld %22.15e %22.15e\n", i, j, s->mind[i*nb+j].step,
                    s->mind[i*nb+j].d, s->mind[i*nb+j].lim);
        }
    }
    fclose(f);
    return GOOD_RETURN;
}

/* Calcula y guarda las posiciones, radio, velocidad heliocentricos y momento angular
 */
int graba_coord_helio(System *s, char *procdir)
{
    FILE *f;
    char fname[101];
    long ncycle, ind, preg, ureg;
    int i;
    double dx, dy, dz, dvx, dvy, dvz, lx, ly, lz;

    for (ncycle=s->nfcycle; ncycle<s->ncycles; ncycle++) {
        for (i=1; i< s->nbodies; i++) {
            snprintf(fname, 100, "%s/ch_%s.%04ld.dat", procdir, s->astro[i].nombre, ncycle);
            f = fopen(fname, "w");
            if (f == NULL) {
                fprintf(stderr, "Error al crear %s\n", fname);
                return BAD_RETURN;
            }
  
            preg = s->nsteps * (ncycle - s->nfcycle);
            ureg = preg + s->nsteps;
            for (ind=preg; ind<ureg; ind++) {
                if (s->astro[i].sd[ind].pos.x == 0.0) break; // planeta vaporizado
                dx = s->astro[i].sd[ind].pos.x - s->astro[0].sd[ind].pos.x;
                dy = s->astro[i].sd[ind].pos.y - s->astro[0].sd[ind].pos.y;
                dz = s->astro[i].sd[ind].pos.z - s->astro[0].sd[ind].pos.z;
                fprintf(f, "%ld %22.15e %22.15e %22.15e %22.15e", s->step[ind],
                        dx, dy, dz, sqrt(dx*dx + dy*dy + dz*dz));
                dvx = s->astro[i].sd[ind].v.x - s->astro[0].sd[ind].v.x;
                dvy = s->astro[i].sd[ind].v.y - s->astro[0].sd[ind].v.y;
                dvz = s->astro[i].sd[ind].v.z - s->astro[0].sd[ind].v.z;
                fprintf(f, " %22.15e %22.15e %22.15e %22.15e",
                        dvx, dvy, dvz, sqrt(dvx*dvx + dvy*dvy + dvz*dvz));
                lx = s->astro[i].m * (dy * dvz - dz * dvy);
                ly = s->astro[i].m * (dz * dvx - dx * dvz);
                lz = s->astro[i].m * (dx * dvy - dy * dvx);
                fprintf(f, " %22.15e %22.15e %22.15e %22.15e\n",
                        lx, ly, lz, sqrt(lx*lx + ly*ly + lz*lz));
                
            }
            fclose(f);
        }
    }
  
    return GOOD_RETURN;
}

/* Crea un fichero concatenado de texto
 */
int concatena_txt(System *s, char *datadir, char *patron, char *procdir, char *fnout)
{
    FILE *fin, *fout;
    char buf[301];
    char fname[101];
    char format[101];
    long ncycle;

    snprintf(fname, 100, "%s/%s", procdir, fnout);
    fout = fopen(fname, "w");
    if (fout == NULL) {
        fprintf(stderr, "Error al crear %s\n", fname);
        return BAD_RETURN;
    }

    snprintf(format, 100, "%s/%s", "%s", patron);
  
    for (ncycle=s->nfcycle; ncycle<s->ncycles; ncycle++) {
        snprintf(fname, 100, format, datadir, ncycle);
        fin = fopen(fname, "r");
        if (fin == NULL) {
            fprintf(stderr, "Error al abrir %s\n", fname);
            return BAD_RETURN;
        }
        while (fgets(buf, 300, fin) != NULL) {
            fputs(buf, fout);
        }
        fclose(fin);
    }
    
    fclose(fout);
  
    return GOOD_RETURN;
}

/* Programa principal
 */
int main(int argc, char **argv)
{
    System *s;
    long ncycles = 0;
    long nfcycle = 0;
    long nsteps = 0;
    char *filedef, *datadir, *procdir;

    printf("%s\n\n", VERSION);
    
    if (argc < 4) {
        fprintf(stderr, "Ejecutar con ./extract_sim filedef datadir procdir [ncycles nsteps nfcycle]\n");
        return 1;
    }
  
    filedef = argv[1];
    datadir = argv[2];
    procdir = argv[3];
    if (argc >= 7) {
        ncycles = atol(argv[4]);
        nsteps = atol(argv[5]);
        nfcycle = atol(argv[6]);
    }

    //printf("%ld %ld %ld\n", ncycles, nfcycle, nsteps);
  
    s = crea_planet_system_de_filedef(filedef, ncycles, nfcycle, nsteps);
    if (s == NULL) {
        printf("Error al crear system\n");
        return BAD_RETURN;
    }

    //printf("%ld %ld %ld\n", s->ncycles, s->nfcycle, s->nsteps);
  
    if (carga_datos(s, datadir, 0) != GOOD_RETURN) {
        printf("Error al cargar datos\n");
        return BAD_RETURN;
    }
  
    //muestra_system(s);
  
    if (graba_datos_orbitas(s, procdir) != GOOD_RETURN) {
        return BAD_RETURN;
    }
  
    if (graba_mind2(s, datadir, procdir) != GOOD_RETURN) {
        return BAD_RETURN;
    }
  
    if (graba_coord_helio(s, procdir) != GOOD_RETURN) {
        return BAD_RETURN;
    }
    
    if (concatena_txt(s, datadir, "dglobal.%04ld.dat", procdir, FILEGCON) != GOOD_RETURN) {
        return BAD_RETURN;
    }
  
    return GOOD_RETURN;
}
