/* SIMulador Sistema PLAnetario, M.Alvarez 2018
 * Trabajo para el TFM
 * del Master de Astrofísica de la UAM
 * 
 * Copyright (c) 2018 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of simspla
 *
 * simspla is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * simspla is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with rebound.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "simspla.h"

/* Crea el Sistema planetario
 */
static System * crea_system(char *nombre, int units, int nbodies, double stepdt,
                            double t0, double eini, long nsave, long nsteps,
                            long ncycles, long nfcycle, double elost_acu,
                            double m0_acu, double mul_rh, double tideson,
                            long showpro, int integrator)
{
    System *s;
    long mem;
    int i;
  
    if ((nsteps % nsave) != 0) {
        fprintf(stderr, "Error, NSTEPS debe ser múltiplo de SAVESTEP\n");
        return NULL;
    }
  
    s = (System *)malloc(sizeof(System));
    if (s == NULL) {
        fprintf(stderr, "Error de memoria al crear System\n");
        return NULL;
    }
  
    s->astro = (Body *)malloc(nbodies * sizeof(Body));
    if (s->astro == NULL) {
        fprintf(stderr, "Error de memoria al crear array de astros\n");
        free(s);
        return NULL;
    }
  
    s->nregs = nsteps / nsave;
    s->rege = (RegDGlobal *)malloc((s->nregs+1) * sizeof(RegDGlobal));
    if (s->rege == NULL) {
        fprintf(stderr, "Error de memoria al crear registros de datos globales\n");
        free(s->astro);
        free(s);
        return NULL;
    }
  
    s->nreg_dist = 0;
    for (i=nbodies-1; i>0; i--) s->nreg_dist += i;
    s->rdist = (RegDist *)malloc(s->nreg_dist * sizeof(RegDist));
    if (s->rdist == NULL) {
        fprintf(stderr, "Error de memoria al crear registros de distancia\n");
        free(s->rege);
        free(s->astro);
        free(s);
        return NULL;
    }
    for (i=0; i<s->nreg_dist; i++) {
        s->rdist[i].dmin = 1e99;
        s->rdist[i].tmin = 0;
    }

    s->nombre = strdup(nombre);
    s->units = units;
    s->integrator = integrator;
    if (units == SU_KMSKG) s->G = G_KMSKG;
    else if (units == SU_AUDAYKG) s->G = G_AUDAYKG;
    else if (units == SU_AUDAYMS) s->G = G_AUDAYMS;
    s->nbodies = nbodies;
    s->nbcforce = nbodies; // por defecto
    s->stepdt = stepdt;
    s->t0 = t0;
    s->eini = eini;
    s->elost_acu = elost_acu;
    s->m0_acu = m0_acu;
    s->mul_rh = mul_rh;
    s->nsteps = nsteps;
    s->nsave = nsave;
    s->showpro = showpro;
    s->ncycles = ncycles;
    s->nfcycle = nfcycle;
    s->mre_active = 0;
    s->mre_fname = NULL;
    s->mre = NULL;
    s->tideson = tideson;
  
    mem = ((s->nbodies * sizeof(RegStep)) + sizeof(RegDGlobal)) * (s->nregs + 1);
    printflog("Creado System, memoria estimada %ld\n", mem);
  
    return s;
}

/* Inicia los datos de un astro
 */
static int inibody(char *nombre, int tipo, int estado, double masa, double radio,
                   double x, double y, double z, double vx, double vy, double vz,
                   long nregs, Body *b)
{
    b->reg = (RegStep *)malloc((nregs+1) * sizeof(RegStep));
    if (b->reg == NULL) return BAD_RETURN;
  
    b->nombre = strdup(nombre);
    b->tipo = tipo;
    b->estado = estado;
    b->masa = masa;
    b->radio = radio;
    b->pos.x = x;
    b->pos.y = y;
    b->pos.z = z;
    b->v.x = vx;
    b->v.y = vy;
    b->v.z = vz;
    b->tmode = TMODE_NOTIDES;

    b->reg[0].pos = b->pos;
    b->reg[0].v = b->v;
  
    return GOOD_RETURN;
}

static void addtidaldata_fkt(double k, double tl, Body *b)
{
    b->tmode = TMODE_FIXED;
    b->k = k;
    b->tl = tl;
}

static void addtidaldata_fks(double k, double sig, double g, Body *b)
{
    double r;
  
    b->tmode = TMODE_FIXED;
    b->k = k;
    b->sig = sig;
    r = b->radio;
    b->tl = (3.0 * r * r * r * r * r * sig) / (2.0 * g * k);
}

static void addtidaldata_vks(double k, double sig, double fr, double g, Body *b)
{
    double r;
  
    b->tmode = TMODE_EVOL;
    b->k = k;
    b->sig = sig;
    b->fr = fr;
    r = b->radio;
    b->tl = (3.0 * r * r * r * r * r * sig) / (2.0 * g * k);
    b->presig = ((3.0 * sig) / (2.0 * g)) / (fr * k * b->tl);
}

static void addtidaldata_vkts(double k, double tl, double sig, double fr, double g, Body *b)
{
    b->tmode = TMODE_EVOL;
    b->k = k;
    b->sig = sig;
    b->fr = fr;
    b->tl = tl;
    b->presig = ((3.0 * sig) / (2.0 * g)) / (fr * k * tl);
}

/* Devuelve el tipo entero de un astro a partir del tipo ascii
 */
static int body_type(char c)
{
    if (c == 'E') return BT_STAR;
    if (c == 'P') return BT_PLANET;
    if (c == 'A') return BT_ASTEROID;
    if (c == 'S') return BT_SATELLITE;
    if (c == 'C') return BT_COMET;
    return BT_ASTEROID;
}

/* Devuelve el tipo ascii de un astro a partir del tipo entero
 */
static char reverse_body_type(int i)
{
    if (i == BT_STAR) return 'E';
    if (i == BT_PLANET) return 'P';
    if (i == BT_ASTEROID) return 'A';
    if (i == BT_SATELLITE) return 'S';
    if (i == BT_COMET) return 'C';
    return 'A';
}

/* Crea sistema planetario a partir de un fichero de definición
 */
System * genera_system_from_filedef(char *fname)
{
    int units = SU_KMSKG;
    int integrator = INTG_WHFAST;
    double stepdt = 1;
    long nsave = 1;
    long showpro = 1;
    long nsteps = 1;
    long ncycles = 1;
    long nfcycle = 0;
    double t0 = 0;
    double eini = 0;
    double elost_acu = 0;
    double m0_acu = 0;
    double mul_rh = 1;
    double tideson = 0;
    int nbodies = 0;
    int tipo, estado;
    double radio, masa, x, y, z, vx, vy, vz;
    double k, tl, sig, fr;
  
    ParFsal pfsal = {".", 0, 'T', 0, 'T', 0, 0, "simspla.log", 1};

    FILE *f;
    System *s = NULL;
    char buf[301], *bufp;
    int inisys = 0;
    int indn1 = 0;
    char *nombre, *tmode, *token;
    int ret, len, nbody;
  
    f = fopen(fname, "r");
    if (f == NULL) {
        fprintf(stderr,"Error al abrir %s\n", fname);
        return NULL;
    }
  
    while (fgets(buf, 300, f) != NULL) {
        if (buf[0] == '#' || buf[0] == '\0') continue;
        bufp = buf;
        while (*bufp == ' ') bufp++; // elimina blancos

        if (!strncmp(bufp,"UNITS=",6)) {
            if (!strncmp(&(bufp[6]),"KMSKG",5)) units = SU_KMSKG;
            else if (!strncmp(&(bufp[6]),"AUDAYKG",7)) units = SU_AUDAYKG;
            else if (!strncmp(&(bufp[6]),"AUDAYMS",7)) units = SU_AUDAYMS;
            else {
                fprintf(stderr, "Parámetro UNITS mal definido\n");
                return NULL;
            }
        }
        if (!strncmp(bufp,"INTEGRATOR=",11)) {
            if (!strncmp(&(bufp[11]),"WHFAST",6)) integrator = INTG_WHFAST;
            else if (!strncmp(&(bufp[11]),"DKD",3)) integrator = INTG_DKD;
            else {
                fprintf(stderr, "Parámetro INTEGRATOR mal definido\n");
                return NULL;
            }
        }
        if (!strncmp(bufp,"STEPDT=",7)) stepdt = atof(&(bufp[7]));
        if (!strncmp(bufp,"SAVESTEP=",9)) nsave = atol(&(bufp[9]));
        if (!strncmp(bufp,"NSTEPS=",7)) nsteps = atol(&(bufp[7]));
        if (!strncmp(bufp,"NCYCLES=",8)) ncycles = atol(&(bufp[8]));
        if (!strncmp(bufp,"NFCYCLE=",8)) nfcycle = atol(&(bufp[8]));
        if (!strncmp(bufp,"T0=",3)) t0 = atof(&(bufp[3]));
        if (!strncmp(bufp,"EINI=",5)) eini = atof(&(bufp[5]));
        if (!strncmp(bufp,"ELOSTACU=",9)) elost_acu = atof(&(bufp[9]));
        if (!strncmp(bufp,"M0ACU=",6)) m0_acu = atof(&(bufp[6]));
        if (!strncmp(bufp,"MULRH=",6)) mul_rh = atof(&(bufp[6]));
        if (!strncmp(bufp,"TIDESON=",8)) tideson = atof(&(bufp[8]));

        if (!strncmp(bufp,"DATADIR=",8)) {
            pfsal.datadir = strdup(&(bufp[8]));
            len = strlen(pfsal.datadir);
            if (len > 0  && pfsal.datadir[len-1] == '\n') pfsal.datadir[len-1] = '\0';
        }
        if (!strncmp(bufp,"GENBDAT,",8)) {
            pfsal.genbdata = 1;
            pfsal.bdataformat = 'T';
            token = strtok(&(bufp[8]), ",");
            if (token[0] == 'b' || token[0] == 'B') pfsal.bdataformat = 'B';
            if (token[0] == 'a' || token[0] == 'A') pfsal.bdataformat = 'A';
        }
        if (!strncmp(bufp,"GENGDAT,",8)) {
            pfsal.gengdata = 1;
            pfsal.gdataformat = 'T';
            token = strtok(&(bufp[8]), ",");
            if (token[0] == 'b' || token[0] == 'B') pfsal.gdataformat = 'B';
            if (token[0] == 'a' || token[0] == 'A') pfsal.gdataformat = 'A';
        }
        if (!strncmp(bufp,"GENMDIS",7)) pfsal.genmindist = 1;
        if (!strncmp(bufp,"GENFDEF",7)) pfsal.gendeffiles = 1;

        if (!strncmp(bufp,"LOGFILE=",8)) {
            pfsal.logfile = strdup(&(bufp[8]));
            len = strlen(pfsal.logfile);
            if (len > 0  && pfsal.logfile[len-1] == '\n') pfsal.logfile[len-1] = '\0';
        }
        if (!strncmp(bufp,"SHOWLCON=",9)) {
            if (bufp[9]=='Y' || bufp[9]=='y') pfsal.showlcon = 1;
            else pfsal.showlcon = 0;
        }
        if (!strncmp(bufp,"SHOWPRO=",8)) showpro = atol(&(bufp[8]));

        if (!inisys && !strncmp(bufp,"SYSTEM,",7)) {
            if (!pfsal.genbdata && !pfsal.gengdata &&
                !pfsal.genmindist && !pfsal.gendeffiles) {
                fprintf(stderr, "No se ha pedido generar ficheros\n");
                return NULL;
            }
            inisys = 1;
            indn1 = 0;
            nombre = strtok(&(bufp[7]), ",");
            token = strtok(NULL, ",");
            nbodies = atoi(token);
            s = crea_system(nombre, units, nbodies, stepdt, t0, eini, nsave,
                            nsteps, ncycles, nfcycle, elost_acu, m0_acu, mul_rh,
                            tideson, showpro, integrator);
            if (s == NULL) {
                return NULL;
            }
            s->pfsal = pfsal;
        }

        if (inisys && !strncmp(bufp,"SMALL,",5)) {
            if (indn1 < nbodies) {
                s->nbcforce = indn1;
            }
        }

        if (inisys && !strncmp(bufp,"BODY,",5)) {
            nombre = strtok(&(bufp[5]), ",");
            token = strtok(NULL, ",");
            tipo = body_type(token[0]);
            token = strtok(NULL, ",");
            estado = atoi(token);
            token = strtok(NULL, ",");
            masa = atof(token);
            token = strtok(NULL, ",");
            radio = atof(token);
            token = strtok(NULL, ",");
            x = atof(token);
            token = strtok(NULL, ",");
            y = atof(token);
            token = strtok(NULL, ",");
            z = atof(token);
            token = strtok(NULL, ",");
            vx = atof(token);
            token = strtok(NULL, ",");
            vy = atof(token);
            token = strtok(NULL, ",");
            vz = atof(token);
            if (indn1 < nbodies) {
                ret = inibody(nombre, tipo, estado, masa, radio, x, y, z,
                              vx, vy, vz, s->nregs, &(s->astro[indn1]));
                if (ret == BAD_RETURN) {
                    fprintf(stderr, "Error al crear cuerpo %s\n", nombre);
                    return NULL;
                }
                indn1++;
            }
        }

        if (inisys && !strncmp(bufp,"PTIDES,",7)) {
            nombre = strtok(&(bufp[7]), ",");
            nbody = find_body_by_name(s, nombre);
            if (nbody < 0) {
                fprintf(stderr, "Error al asignar datos de fuerzas de marea al cuerpo %s\n", nombre);
                return NULL;
            }
            tmode = strtok(NULL, ",");
            if (strcmp(tmode, "FKT") == 0) {
                token = strtok(NULL, ",");
                k = atof(token);
                token = strtok(NULL, ",");
                tl = atof(token);
                addtidaldata_fkt(k, tl, &(s->astro[nbody]));
            } else if (strcmp(tmode, "FKS") == 0) {
                token = strtok(NULL, ",");
                k = atof(token);
                token = strtok(NULL, ",");
                sig = atof(token);
                addtidaldata_fks(k, sig, s->G, &(s->astro[nbody]));
            } else if (strcmp(tmode, "VKS") == 0) {
                token = strtok(NULL, ",");
                k = atof(token);
                token = strtok(NULL, ",");
                sig = atof(token);
                token = strtok(NULL, ",");
                fr = atof(token);
                addtidaldata_vks(k, sig, fr, s->G, &(s->astro[nbody]));
            } else if (strcmp(tmode, "VKTS") == 0) {
                token = strtok(NULL, ",");
                k = atof(token);
                token = strtok(NULL, ",");
                tl = atof(token);
                token = strtok(NULL, ",");
                sig = atof(token);
                token = strtok(NULL, ",");
                fr = atof(token);
                addtidaldata_vkts(k, tl, sig, fr, s->G, &(s->astro[nbody]));
            } else {
                fprintf(stderr, "Error en modo de fuerzas de marea del cuerpo %s\n", nombre);
                return NULL;
            }
        }

        if (inisys && !strncmp(bufp,"EVMR,",5)) {
            s->mre_fname = strdup(strtok(&(bufp[5]), ","));
            token = strtok(NULL, ",");
            s->mre_t0 = atof(token);
            token = strtok(NULL, ",");
            s->mre_utsim = atof(token);
            token = strtok(NULL, ",");
            s->mre_evstep = atol(token);
        }
    }

    if (indn1 != nbodies) {
        fprintf(stderr, "Error, sólo definidos %d astros de %d\n", indn1, nbodies);
        return NULL;
    }
  
    return s;
}

/* Guarda fichero de definición para poder rearrancar la simulación
 * tras la finalización de un ciclo
 */
int guarda_system_to_filedef(System *s, long ncycle)
{
    FILE *f;
    char fname[101];
    char *sunit, *sintegrator;
    RegStep *rs;
    int i;
    
    snprintf(fname, 100, "%s/cycle.%04ld.def", s->pfsal.datadir, ncycle);
    f = fopen(fname, "w");
    if (f == NULL) {
        fprintf(stderr,"Error al crear %s\n", fname);
        return BAD_RETURN;
    }

    sunit = "KMSKG";
    if (s->units == SU_AUDAYKG) sunit = "AUDAYKG";
    else if (s->units == SU_AUDAYMS) sunit = "AUDAYMS";
    sintegrator = "WHFAST";
    if (s->integrator == INTG_DKD) sintegrator = "DKD";
    fprintf(f, "UNITS=%s\n", sunit);
    fprintf(f, "INTEGRATOR=%s\n", sintegrator);
    fprintf(f, "STEPDT=%f\n", s->stepdt);
    fprintf(f, "SAVESTEP=%ld\n", s->nsave);
    fprintf(f, "NSTEPS=%ld\n", s->nsteps);
    fprintf(f, "NCYCLES=%ld\n", s->ncycles);
    fprintf(f, "NFCYCLE=%ld\n", ncycle);
    fprintf(f, "T0=%.17e\n", s->t0 + (s->stepdt * s->nsteps * (ncycle - s->nfcycle)));
    fprintf(f, "EINI=%.17e\n", s->eini);
    fprintf(f, "ELOSTACU=%.17e\n", s->elost_acu);
    fprintf(f, "M0ACU=%.17e\n", s->m0_acu);
    fprintf(f, "MULRH=%.17e\n", s->mul_rh);
    fprintf(f, "TIDESON=%.17e\n", s->tideson);

    fprintf(f, "DATADIR=%s\n", s->pfsal.datadir);
    if (s->pfsal.genbdata) fprintf(f, "GENBDAT,%c\n", s->pfsal.bdataformat);
    if (s->pfsal.gengdata) fprintf(f, "GENGDAT,%c\n", s->pfsal.gdataformat);
    if (s->pfsal.genmindist) fprintf(f, "GENMDIS\n");
    if (s->pfsal.gendeffiles) fprintf(f, "GENFDEF\n");
  
    fprintf(f, "LOGFILE=%s\n", s->pfsal.logfile);
    fprintf(f, "SHOWLCON=%c\n", s->pfsal.showlcon ? 'Y' : 'N');
    fprintf(f, "SHOWPRO=%ld\n", s->showpro);

    fprintf(f, "SYSTEM,%s,%d\n", s->nombre, s->nbodies);
    for (i=0; i<s->nbodies; i++) {
        if (i==s->nbcforce)
            fprintf(f, "SMALL\n");
        rs = &(s->astro[i].reg[s->nregs]);
        fprintf(f, "BODY,%s,%c,%d,%.17e,%.17e,%.17e,%.17e,%.17e,%.17e,%.17e,%.17e\n",
                s->astro[i].nombre, reverse_body_type(s->astro[i].tipo),s->astro[i].estado,
                s->astro[i].masa, s->astro[i].radio, rs->pos.x, rs->pos.y, rs->pos.z,
                rs->v.x, rs->v.y, rs->v.z);
    }

    for (i=0; i<s->nbodies; i++) {
        if (s->astro[i].tmode == TMODE_FIXED) {
            fprintf(f, "PTIDES,%s,FKT,%.17e,%.17e\n",
                    s->astro[i].nombre, s->astro[i].k, s->astro[i].tl);
        } else if (s->astro[i].tmode == TMODE_EVOL) {
            fprintf(f, "PTIDES,%s,VKTS,%.17e,%.17e,%.17e,%.17e\n",
                    s->astro[i].nombre, s->astro[i].k, s->astro[i].tl,
                    s->astro[i].sig, s->astro[i].fr);
        }
    }

    if (s->mre_active) {
        fprintf(f, "EVMR,%s,%.17e,%.17e,%ld\n", s->mre_fname, s->mre_t0,
                s->mre_utsim, s->mre_evstep);
    }
  
    fclose(f);
  
    return GOOD_RETURN;
}

/* Destruye y libera memoria de un sistema planetario
 */
void destruye_system(System *s)
{
    int i;

    for (i=0; i<s->nbodies; i++) {
        free(s->astro[i].nombre);
        free(s->astro[i].reg);
    }
    free(s->nombre);
    free(s->pfsal.datadir);
    free(s->pfsal.logfile);
    free(s->astro);
    free(s->rege);
    free(s->rdist);
    if (s->mre_active) {
        free(s->mre_fname);
        free(s->mre->range);
        free(s->mre);
    }
    free(s);
}

/* Busca el indice de un astro por su nombre
 * devuelve -1 si no lo encuentra
 */
int find_body_by_name(System *s, char *name)
{
    int i;
    
    for (i=0; i<s->nbodies; i++) {
        if (strcmp(s->astro[i].nombre, name) == 0)
            return i;
    }
  
    return -1;
}
