/* SIMulador Sistema PLAnetario, M.Alvarez 2018
 * Trabajo para el TFM
 * del Master de Astrofísica de la UAM
 * 
 * Copyright (c) 2018 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of simspla
 *
 * simspla is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * simspla is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with rebound.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "simspla.h"
#include "rebound.h"
#include "myrtools.h"

/* Mi versión de reb_output_timing para que funcione bien con los ciclos
 */
void my_reb_output_timing(struct reb_simulation* r, const double tini, const double tmax)
{
    const int N = r->N;
    int N_tot = N;
    struct timeval tim;
    gettimeofday(&tim, NULL);
    double temp = tim.tv_sec+(tim.tv_usec/1000000.0);
    if (r->output_timing_last==-1){
        r->output_timing_last = temp;
    }else{
        printf("\r");
    }
    printf("N_tot= %- 9d  ",N_tot);
    if (r->integrator==REB_INTEGRATOR_SEI){
        printf("t= %- 9f [orb]  ",r->t*r->ri_sei.OMEGA/2./M_PI);
    }else{
        printf("t= %- 9f  ",r->t);
    }
    printf("dt= %- 9f  ",r->dt);
    printf("cpu= %- 9f [s]  ",temp-r->output_timing_last);
    if (tmax>0){
        printf("t/tmax= %5.2f%%",(r->t-tini)/(tmax-tini)*100.0);
    }
    fflush(stdout);
    r->output_timing_last = temp;
}

/* Calcula la parte de la energía debida sólo al cuerpo principal (indice 0)
 */
double my_reb_tools_energy_a0(const struct reb_simulation* const r)
{
    System *s;
    struct reb_particle *pi, *pj;

    double e_kin = 0.;
    double e_pot = 0.;

    s = (System *)(r->extras);

    pi = &(r->particles[0]);
    e_kin += 0.5 * pi->m * (pi->vx*pi->vx + pi->vy*pi->vy + pi->vz*pi->vz);
    
    for (int j=1;j<s->nbodies;j++){
        if (s->astro[j].estado != 0) continue;
        pj = &(r->particles[s->astro[j].indrb]);
        e_pot -= r->G * pj->m * pi->m / s->rdist[j-1].d;
    }

    return e_kin + e_pot;
}

/* Calcula distancias y guarda las mínimas
 */
void calcula_dist(const struct reb_simulation* const r)
{
    System *s;
    struct reb_particle *pi, *pj;
    int i, j, ind;
    double dx, dy, dz, dist;

    s = (System *)(r->extras);

    ind = 0;
    for (i=0; i<s->nbodies-1; i++) {
        if (s->astro[i].estado == 0) {
            pi = &(r->particles[s->astro[i].indrb]);
            for (j=i+1; j<s->nbodies; j++) {
                if (s->astro[j].estado == 0) {
                    pj = &(r->particles[s->astro[j].indrb]);
                    dx = pi->x - pj->x;
                    dy = pi->y - pj->y;
                    dz = pi->z - pj->z;
                    dist = sqrt(dx * dx + dy * dy + dz * dz);
                    s->rdist[ind].d = dist;
                    if (dist < s->rdist[ind].dmin) {
                        s->rdist[ind].dmin = dist;
                        s->rdist[ind].tmin = r->t;
                        if (i>0) s->rdist[ind].dlim =
                            (s->astro[i].hillr + s->astro[j].hillr) * s->mul_rh;
                    }
                }
                ind++;
            }
        } else {
            ind += s->nbodies - 1 - i;
        }
    }
}

/* Calcula los radios de Hill de los planetas
 */
void calcula_hillr(const struct reb_simulation* const r)
{
    System *s;
    struct reb_particle *p0, *pi;
    int i;
    double dvx, dvy, dvz, vel2, mu, vc2, a;

    s = (System *)(r->extras);

    p0 = &(r->particles[0]);
    for (i=1; i<s->nbodies; i++) {
        if (s->astro[i].estado == 0) {
            pi = &(r->particles[s->astro[i].indrb]);
            dvx = p0->vx - pi->vx;
            dvy = p0->vy - pi->vy;
            dvz = p0->vz - pi->vz;
            vel2 = dvx*dvx + dvy*dvy + dvz*dvz;
            mu = r->G * (p0->m + pi->m);
            vc2 = mu / s->rdist[i-1].d;
            a = -mu / (vel2 - 2.0 * vc2); // semi-eje mayor
            if (a<=0.0) a = s->rdist[i-1].d; // órbita hiperbólica
            s->astro[i].hillr = a * pow(((1.0/3.0)*pi->m/p0->m), (1.0/3.0));
        }
    }
}

/* Planeta alcanzado por el radio del Sol
 */
void vaporiza_planeta(struct reb_simulation* r, int ip)
{
    System *s;
    int j, ind;
    double msum, tmp1, tmp2;

    s = (System *)(r->extras);
    
    ind = s->astro[ip].indrb;
    s->astro[ip].indrb = -1;
    s->astro[ip].estado = -1;

    //printf("%.15e %.15e %.15e ", r->particles[0].x, r->particles[0].y, r->particles[0].z);
    //printf("%.15e %.15e %.15e\n", r->particles[0].vx, r->particles[0].vy, r->particles[0].vz);
    
    msum = r->particles[0].m + r->particles[ind].m;
    tmp1 = r->particles[0].m / msum;
    tmp2 = r->particles[ind].m / msum;
    r->particles[0].x = r->particles[0].x * tmp1 + r->particles[ind].x * tmp2;
    r->particles[0].y = r->particles[0].y * tmp1 + r->particles[ind].y * tmp2;
    r->particles[0].z = r->particles[0].z * tmp1 + r->particles[ind].z * tmp2;
    r->particles[0].vx = r->particles[0].vx * tmp1 + r->particles[ind].vx * tmp2;
    r->particles[0].vy = r->particles[0].vy * tmp1 + r->particles[ind].vy * tmp2;
    r->particles[0].vz = r->particles[0].vz * tmp1 + r->particles[ind].vz * tmp2;

    //printf("%.15e %.15e %.15e ", r->particles[0].x, r->particles[0].y, r->particles[0].z);
    //printf("%.15e %.15e %.15e\n", r->particles[0].vx, r->particles[0].vy, r->particles[0].vz);
    
    s->m0_acu += r->particles[ind].m;
    r->particles[0].m += r->particles[ind].m;
    reb_remove(r, ind, 1);

    for (j=ip+1; j<s->nbodies; j++) {
        if (s->astro[j].estado != 0) continue;
        s->astro[j].indrb -= 1;
    }
}
